function theResult = batch(theBatchFile, theVerboseFlag)

% batch/batch -- Constructor for "batch" class.
%  batch('theBatchFile') runs theBatchFile, whose
%   name is requested by the "uigetfile" dialog
%   if a wildcard argument is given.  Only a single
%   batch thread can be run.
%  batch([]) terminates the current batch monitor.
%   This happens automatically at end-of-file.
%  batch('version') displays the current version
%   of the batch system.
%  batch (no argument) returns the batch-monitor,
%   but only if an output argument is provided.
%   Otherwise, help is shown.
%
% Public Methods:
%  batch('filename')	Start running a batch-file.
%  batch([])			Deactivate batch processing.
%  running(batch)		TRUE if running batch.
%  get(batch)			Get next line of batch data.
%  lineno(batch)        Lines processed.
 
% Copyright (C) 1999 Dr. Charles R. Denham, ZYDECO.
%  All Rights Reserved.
%   Disclosure without explicit written consent from the
%    copyright owner does not constitute publication.
 
% Version of 18-Oct-1999 11:52:14.
% Updated    19-Oct-1999 10:26:56.

persistent BATCH_MONITOR

% Help.

if nargout < 1 & nargin < 1
	help(mfilename)
	return
end

% Increment or return batch-file line-number.

if nargin > 0
	if isequal(theBatchFile, '__nextlineno__')
		self = BATCH_MONITOR;
		self.itsLineNumber = self.itsLineNumber + 1;
		BATCH_MONITOR = self;
		return
	elseif isequal(theBatchFile, '__lineno__')
		self = BATCH_MONITOR;
		result = self.itsLineNumber;
		if nargout > 0
			theResult = result;
		else
			assignin('caller', 'ans', result)
		end
		return
	elseif isequal(theBatchFile, 'version')
		help batch/version
		return
	end
end

% Constructor.

if isempty(BATCH_MONITOR)
	theStruct.itsBatchFile = '';
	theStruct.itsBatchFilePtr = [];
	theStruct.itsLineNumber = [];
	theStruct.itIsRunning = any(0);
	theStruct.itIsVerbose = any(0);
	self = class(theStruct, 'batch');
	BATCH_MONITOR = self;
end

% Destructor.

if nargin > 0 & isempty(theBatchFile)
	self = BATCH_MONITOR;
	if ~isempty(self.itsBatchFilePtr)
		status = fclose(self.itsBatchFilePtr);
		if status == 0
			BATCH_MONITOR = [];
		end
	end
	if nargout > 0
		theResult = BATCH_MONITOR;
	else
		assignin('caller', 'ans', BATCH_MONITOR)
	end
	return
end

% Run a batch-file.

if nargout < 1
	if nargin > 0 & any(theBatchFile == '*')
		[theFile, thePath] = uigetfile(theBatchFile, 'Select A Batch File:');
		if ~any(theFile), return, end
		if thePath(end) ~= filesep, thePath(end+1) = filesep; end
		theBatchFile = [thePath theFile];
	end
	fp = fopen(theBatchFile, 'r');
	if fp < 0
		disp([' ## Unable to open batch file "' theBatchFile '"'])
		return
	end
	self = BATCH_MONITOR;
	self.itsBatchFile = theBatchFile;
	self.itsBatchFilePtr = fp;
	self.itsLineNumber = 0;
	self.itIsRunning = any(1);
	if nargin < 2, theVerboseFlag = 0; end
	if ischar(theVerboseFlag), theVerboseFlag = eval(theVerboseFlag); end
	self.itIsVerbose = logical(theVerboseFlag);
	BATCH_MONITOR = self;
	run(BATCH_MONITOR)
end

if nargout > 0
	theResult = BATCH_MONITOR;
end
