function premask(theDataFile, theMaskFile, vel, cor, agc, pgd)

% premask -- Pre-mask for "Starbear" interaction.
%  premask('theDataFile', 'theMaskFile', vel, cor, agc, good)
%   marks 'theMaskFile' in accordance with the acceptable ranges
%   [min max] for the "vel", "cor", "AGC", and "PGd" variables
%   in 'theDataFile', respectively.  Velocity is assumed to be
%   given in cm/s, even though it is stored as mm/s in the file.
 
% Copyright (C) 1999 Dr. Charles R. Denham, ZYDECO.
%  All Rights Reserved.
%   Disclosure without explicit written consent from the
%    copyright owner does not constitute publication.
 
% Version of 14-Jan-1999 14:25:34.
% edited by JMC and CRD on 20 May 1999
% can now be called by a single line in a function
% CRD is working to allow a 4x2 input for all the ranges rather than 
% the current 1x2.

CHUNK = 100;

if nargin < 1, theDataFile = ''; end
if nargin < 2, theMaskFile = ''; end

if isempty(theDataFile), theDataFile = '*'; end
if isempty(theMaskFile), theMaskFile = '*'; end

if any(theDataFile == '*')
   help(mfilename)
   thePrompt = theDataFile;
   [theFile, thePath] = uigetfile(thePrompt, 'Select ADCP Data File');
   if ~any(theFile), return, end
   if thePath(end) ~= filesep, thePath(end+1) = filesep; end
   theDataFile = [thePath theFile];
   cd(thePath)
end

if any(theMaskFile == '*')
   help(mfilename)
   thePrompt = theMaskFile;
   [theFile, thePath] = uigetfile(thePrompt, 'Select ADCP Mask File');
   if ~any(theFile), return, end
   if thePath(end) ~= filesep, thePath(end+1) = filesep; end
   theMaskFile = [thePath theFile];
   cd(thePath)
end

theData = netcdf(theDataFile, 'nowrite');
if isempty(theData), return, end

theMask = netcdf(theMaskFile, 'write');
if isempty(theMask), close(theData), return, end

isEarth = isequal(lower(theData.transform(:)), 'eart');

if nargin < 3
	vel = [-3276.8 3276.7];
	vel1 = [-3276.8 3276.7];
	vel2 = [-3276.8 3276.7];
	vel3 = [-3276.8 3276.7];
	vel4 = [-3276.8 3276.7];
	cor = [0 255];
	agc = [0 255];
	pgd = [0 100];
	if ~isEarth
		ADCP_Valid_Range.Velocity = vel;
	else
		ADCP_Valid_Range.U = vel1;
		ADCP_Valid_Range.V = vel2;
		ADCP_Valid_Range.W = vel3;
		ADCP_Valid_Range.Werr = vel4;
	end
	ADCP_Valid_Range.Correlation = cor;
	ADCP_Valid_Range.Gain = agc;
	ADCP_Valid_Range.PercentGood = pgd;
	s = uigetinfo(ADCP_Valid_Range);
	if ~isempty(s)
		if ~isEarth
			vel = getinfo(s, 'Velocity');
			vel = ones(4, 1) * vel;
		else
			u = getinfo(s, 'U');
			v = getinfo(s, 'V');
			w = getinfo(s, 'W');
			werr = getinfo(s, 'Werr');
			vel = [u; v; w; werr];
		end
		cor = ones(4, 1) * getinfo(s, 'Correlation');
		agc = ones(4, 1) * getinfo(s, 'Gain');
		pgd = ones(4, 1) * getinfo(s, 'PercentGood');
	end
end

vel = 10 * vel;   % Convert cm/s to mm/s.

f = theData;
g = theMask;

for k = 1:4
	vmask{k} = g{['vel' int2str(k)]};
end

nrecords = size(f{'vel1'});
nrecords = nrecords(1);

% Velocity.

vel

if any(isfinite(vel(:)))
	for k = 1:4
		theVarname = ['vel' int2str(k)];
		disp([' ## ' theVarname ' ...'])
		p = f{theVarname};
		q = vmask{k};
		m = size(p, 1);
		i = 0;
		while i < m
			j = i+1:min(i+CHUNK, nrecords);
			s = p(j, :);
			t = q(j, :);
		%	bad = s < min(vel(k, :)) | s > max(vel(k, :));
			bad = s < min(vel(1, :)) | s > max(vel(1, :));
			t = t | bad;
			q(j, :) = t;
			i = i + CHUNK;
		end
	end
end

% Correlation.

if any(isfinite(cor(:)))
	for k = 1:4
		theVarname = ['cor' int2str(k)];
		disp([' ## ' theVarname ' ...'])
		p = f{theVarname};
		q = vmask{k};
		m = size(p, 1);
		i = 0;
		while i < m
			j = i+1:min(i+CHUNK, nrecords);
			s = p(j, :);
			t = q(j, :);
			bad = s < min(cor(1, :)) | s > max(cor(1, :));
		%		bad = s < min(cor(k, :)) | s > max(cor(k, :));
		t = t | bad;
			q(j, :) = t;
			i = i + CHUNK;
		end
	end
end

% Gain (AGC).

if any(isfinite(agc(:)))
	for k = 1:4
		theVarname = ['AGC' int2str(k)];
		disp([' ## ' theVarname ' ...'])
		p = f{theVarname};
		q = vmask{k};
		m = size(p, 1);
		i = 0;
		while i < m
			j = i+1:min(i+CHUNK, nrecords);
			s = p(j, :);
			t = q(j, :);
	%		bad = s < min(agc(k, :)) | s > max(agc(k, :));
			bad = s < min(agc(1, :)) | s > max(agc(1, :));
			t = t | bad;
			q(j, :) = t;
			i = i + CHUNK;
		end
	end
end

% Percent-good.

if any(isfinite(pgd(:)))
	for k = 1:4
		theVarname = ['PGd' int2str(k)];
		disp([' ## ' theVarname ' ...'])
		p = f{theVarname};
		q = vmask{k};
		m = size(p, 1);
		i = 0;
		while i < m
			j = i+1:min(i+CHUNK, nrecords);
			s = p(j, :);
			t = q(j, :);
%			bad = s < min(pgd(k, :)) | s > max(pgd(k, :));
			bad = s < min(pgd(1, :)) | s > max(pgd(1, :));
			t = t | bad;
			q(j, :) = t;
			i = i + CHUNK;
		end
	end
end

close(g)
close(f)
