
function epDataFile = adcp2ep(adcpFile, epDataFile, ADCPtype, dlgFile)

%function epDataFile = adcp2ep(adcpFile, epDataFile, ADCPtype, dlgFile)
% This function is used to translate RDI ADCP data into variables that are
% in terms of earth coordinates and create an epic compatible data file.
% If the data is in Beam coordinates it will be transformed by runbm2g.m into
% Earth coordinates.  This transformation can be run on workhorse and broad
% band data, but the ADCP type must be specified.
%
% Magnetic Declination
%		If a magnetic declination was provided to the insturment prior to 
%	deployment or to rdi2cdf in post-processing it will be applied at this
%	time for both Earth and Beam coordinat data
%
%Inputs:
%	adcpFile = the ADcp data file in beam coordinates 
%		(Note:if running routines in sequence it should be the trimFile.)
%	epDataFile = the new Epic compatable file that will be created 
%	ADCPtype = WH or BB; will default to WH if not specified
%		WH = workhorse, BB = broad band
%		note: if BB, do not need a dlgFile
%	dlgFile = the dialog file that was created when the ADCP was "deployed"
%
%	Note: If the names of the files are not given, they will be requested.	
%
% Output:
%	epDataFile = same as input


% sub-functions:
%	runbm2g.m
%	ep_time.m
%	gregorian.m

% Written by Jessica M. Cote
% for the U.S. Geological Survey
% Coastal and Marine Geology Program
% Woods Hole, MA
% http://woodshole.er.usgs.gov/
% Please report bugs to jcote@usgs.gov


% version 1.1
%   updated 12-sep=05 (SDR) changed the method in which metadata is
%               collected by the user to a more friendly manner
%   fixed 01-sep-05 (SDR) valid range attribute for velocity data so that
%                   it is now [-1000 1000] as it was formally [1000 1000]
%   updated 12-aug-2005 (SDR) removed fill_value definition in time and
%               time2 EPIC dimensions
%   fixed 30-Apr- 2003 depth calculation for down but no pressure sensor
%   updated 9-Apr-2003  depth calculations to use prssure sensor if available and able to calculate downward looking
%   added note to pressure variable
%   updated so runbm2g history comment will be added to epic file history attributes
%   updated latitude variable so same as longitude variable
%
% updated 19-March-2003 (ALR) added pressure variable
% updated 10-Jan-2003 (ALR) added ';' to stop data stream...
% updated 06-aug-2002 (ALR) Added OCSD to Experiment and LA Shelf ADCP to Description
% updated 01-Jul-2002 (ALR) time conversion won't crash on very short files (line 143)
% updated 29-Apr-2002 (ALR) fixed time stamp error; when moving time stamp to middle of ens, need to add instead of subtract
%       (line 132 corrected)

% version 1.0
% updated 03-Jul-2001 fixed spelling of Buoy (ALR)
% updated 28-Jun-2001 will now work with Matlab 6.0, r.12; problem was with getinfo.m for latitude and
%       longitude values line 428 to 433. (ALR)
% updated 28-Dec-2000 added linefeeds to history/comment attribute (ALR)
% updated 20-Oct-2000 so when computing data in EARTH coordinates the fill values
%		remain the same even after the data is corrected for magnetic declination. ALR
% updated 05-Oct-2000 added additional experiment names and locations.  A. Ramsey
% updated 03-Aug-2000 so creation date will include time.  Fran Hotchkiss
% updated 15-Mar-2000 08:49:03,
%		fixed Pgd and AGC corruption
% updated 02-Feb-2000 09:34:12,
%		fixed temperature weirdness and added rotation for earth data
% updated 16-Dec-1999 12:10:41, put in some clears to help memory
% updated 10-Dec-1999 10:52:22, modified to handel BB
% updated 29-Oct-1999 09:20:26
% updated 19-Oct-1999 13:43:47

%tell us what function is running
Mname=mfilename;
disp('')
disp([ Mname ' is currently running']);

if running(batch)
   adcpFile = get(batch);
   eval(['adcpFile = ' adcpFile ';'])
   epDataFile = get(batch);
   eval(['epDataFile = ' epDataFile ';'])   
   ADCPtype = get(batch);
   eval(['ADCPtype = ' ADCPtype ';'])
else

if nargin < 1, help(mfilename), adcpFile='';, end
if nargin < 2, epDataFile='';, end
if nargin < 3, ADCPtype = '';, end
if nargin < 4, dlgFile='';, end

if isempty(adcpFile), adcpFile = '*'; end
if isempty(epDataFile), epDataFile = '*';, end
if isempty(ADCPtype), ADCPtype = 'WH';, end
if isempty(dlgFile), dlgFile = '*'; end

% Open ADCP beam file.
if any(adcpFile == '*')
	[theFile, thePath] = uigetfile(adcpFile, 'Select ADCP File:');
	if ~any(theFile), return, end
	if thePath(end) ~= filesep, thePath(end+1) = filesep; end
	adcpFile = [thePath theFile];
end

[path,name,ext,ver]=fileparts(adcpFile);
suggest=[name(1:end-1) '.nc'];
%create ADCP Geographic coordinates File
if any(epDataFile == '*')
	[theFile, thePath] = uiputfile(suggest, 'Save Data in geographic coordinates as:');
	if ~any(theFile), return, end
	if thePath(end) ~= filesep, thePath(end+1) = filesep; end
	epDataFile = [thePath theFile];
end

end %if batch

B = netcdf(adcpFile );
if isempty(B), return, end

%let's deal with the time
ppens=B.pings_per_ensemble(:);
tp=B.time_between_ping_groups(:);
%in order to ping as fast as possible tp may be set to 0,
%but in reality it is approximately 0.288 sec per ping.
if isequal(tp,0);
   tp = 0.288;
end

tens=ppens*tp; 

allTIM=B{'TIM'}(:);
% calculate time step for output file, in seconds (FH 10 May 2000)
timsecs=B{'TIM'}(:)*24*3600;
delta=mean(diff(timsecs))%;
clear timsecs

gtD=gregorian(allTIM);
start_time = datestr(datenum(gtD(1,1),gtD(1,2),gtD(1,3),gtD(1,4),gtD(1,5),gtD(1,6)),0);
stop_time = datestr(datenum(gtD(end,1),gtD(end,2),gtD(end,3),gtD(end,4),gtD(end,5),gtD(end,6)),0);

%Tim is the beginning of the ensemble, make it the middle
tsec=gtD(:,4)*3600 + gtD(:,5)*60 + gtD(:,6);
tmid=tsec+tens/2;
disp(['TIM was corrected by ' num2str(tens/2) ' sec = half the ensemble time']);
disp(' ')
   sec=rem(tmid,60);
   hmt=(tmid-sec)/60;
   minu=rem(hmt,60);
   hr=(hmt-minu)/60;
   Mid_Ens_time=[gtD(:,1) gtD(:,2) gtD(:,3) hr minu sec];
   
disp('Converting "TIM" to "time" and "time2"');
[m,n] = size(Mid_Ens_time);
for ii=1:m;
	alltime(ii,:)=ep_time(Mid_Ens_time(ii,:));
end

Time=alltime(:,1);
Time2=alltime(:,2);

%get some information

theFillValue = fillval(B{'vel1'});
bin = size(B('bin'),1);
ensemble = size(B('ensemble'),1);
xducer_off = B{'D'}.xducer_offset_from_bottom (:);   
wdepth = B{'D'}.water_depth (:); 
serial = B.ADCP_serial_number(:);
coord = B.transform(:);
orientation = B.orientation(:);
%defined these 2 varibales earlier since getting screwed up!
TTX = B{'Tx'};
tempC = TTX(:);


%Depth Calculation: (added 04-Apr-2003)
%   1.  Use pressure sensor if available
%   2.  Use values from Surface.exe calculated using Trimbins.m (only for upward looking)
%   3.  Use user input values calculated using Trimbins.m for upward looking or user input values asked for now for downward looking

switch orientation
case 'UP'
%Is there a pressure variable?
if (B{'Pressure'}) ~=0  
    Press = B{'Pressure'}(:);       % pressure at transducer head in pascals
    mnPress = mean(Press);      % average of pressure
    depth_head = mnPress/9806.65;   % depth at transducer head in meters
    center_first_bin = B{'D'}.center_first_bin(:);
    bin_size = B{'D'}.bin_size(:);
    
    depth_head_corrected = depth_head+xducer_off;
    depth = depth_head_corrected - B{'D'}(:);
    dnote = 'Depth values were calculated using the ADCP Pressure Sensor, assuming 9806.65 Pascals per meter.';
    wdepth = depth_head_corrected;
    if ~isempty(B{'height'})
        height = theFillValue*ones(1,ensemble); %Do not need the height variable from surface because the pressure reading is more accurate (ALR 4/4/03)
    end
else
    Press = theFillValue*ones(1,ensemble);
    depth = wdepth - B{'D'}(:);     %wdepth was calculated from surface using trimbins or was a user input value given to trimbins
    %is there a height variable?
    if ~isempty(B{'height'})
        height=B{'height'}(:);
        dnote = 'Depth values were calculated using Surface.exe output';
    else
        height = theFillValue*ones(1,ensemble);
        dnote = 'Depth values were calculated using user input values';
    end
end

case 'DOWN'
   %Is there a pressure variable?
   if (B{'Pressure'}) ~=0
     Press = B{'Pressure'}(:);       % pressure at transducer head in pascals
     mnPress = mean(Press);      % average of pressure
     depth_head = mnPress/9806.65;   % depth at transducer head in meters
     center_first_bin = B{'D'}.center_first_bin(:);
     bin_size = B{'D'}.bin_size(:);
     
    bin1 = depth_head+center_first_bin;
    binEnd = ((bin-1)*bin_size)+bin1; 
    depth = (bin1:bin_size:binEnd)';
    wdepth = depth_head + xducer_off; %water depth is depth at head plus transducer offset.
    dnote = 'Depth values were calculated using the ADCP Pressure Sensor, assuming 9806.65 Pascals per meter.';
    height = theFillValue*ones(1,ensemble);
    
   else
      if running(batch)
          MSL = get(batch);
           eval(['MSL = ' MSL ';'])
           disp(['User input value for depth: ' (num2str(MSL)) ' meters' ])
        else
            disp('User must input the water depth information')
	         Depth_Information.mean_sea_level.value = {0};
   	          Depth_Information.mean_sea_level.units = {'meters'};
	         Depth_Information = uigetinfo(Depth_Information');	
           
   	          %check units
      	       infoD = getinfo(Depth_Information,'mean_sea_level');
                unitD = getinfo(infoD,'units');
              
	       	    if ~isequal(unitD,'meters')
            	    disp('User error!! Depth Information must be in meters')
             	    pause(3)
                  Depth_Information = uigetinfo(Depth_Information)
               end
           
         	  	infoMSL = getinfo(Depth_Information,'mean_sea_level');
                MSL = getinfo(infoMSL,'value');
                 disp(['User input value for depth: ' (num2str(MSL)) ' meters' ])
             end %end if/else batch
             center_first_bin = B{'D'}.center_first_bin(:);
             bin_size = B{'D'}.bin_size(:);
             depth_head = MSL - xducer_off;
             
             bin1 = depth_head+center_first_bin;
             binEnd = ((bin-1)*bin_size)+bin1; 
             depth = (bin1:bin_size:binEnd)';
             dnote = 'Depth values were calculated using user input values';
             Press = theFillValue*ones(1,ensemble);
             height = theFillValue*ones(1,ensemble);
             wdepth = MSL; %water depth equals user input mean sea level
      end %end if pressure/else
end %end switch

disp(['The file has ' num2str(ensemble) ' ensembles and ' num2str(bin) ' bins']);

disp(' ')
disp('Averaging echo intensity')
% Average echo intensity.
i1 = B{'AGC1'};
i1 = autonan(i1,1);
i2 = B{'AGC2'};
i2 = autonan(i2,1);
i3 = B{'AGC3'};
i3 = autonan(i3,1);
i4 = B{'AGC4'};
i4 = autonan(i4,1);
iall(:,:,1) = i1(:,:);
iall(:,:,2) = i2(:,:);
iall(:,:,3) = i3(:,:);
iall(:,:,4) = i4(:,:);
iavg = mean(iall,3);

%free up some memory
clear i1 i2 i3 i4 iall

disp(' ')
disp('Averaging percent good')

switch coord
case 'BEAM'
   
% Average percent good.
p1 = B{'PGd1'};
p1 = autonan(p1,1);
p2 = B{'PGd2'};
p2 = autonan(p2,1);
p3 = B{'PGd3'};
p3 = autonan(p3,1);
p4 = B{'PGd4'};
p4 = autonan(p4,1);
pall(:,:,1) = p1(:,:);
pall(:,:,2) = p2(:,:);
pall(:,:,3) = p3(:,:);
pall(:,:,4) = p4(:,:);
pavg = mean(pall,3);

case 'EARTH'
   p4 = B{'PGd4'};
	p4 = autonan(p4,1);
   pavg = p4(:,:);
end

%free up some memory
clear p1 p2 p3 p4 pall


%check for fill values
	TF = isnan(iavg);
	MTF = max(max(TF));
	if MTF
	   fill_flag = 1;
	end
   
   TF = isnan(pavg);
	MTF = max(max(TF));
	if MTF
   	fill_flag = 1;
	end

%get the velocity data and set the new velocity variable
%dependent on being in Beam or Earth coordinates
switch coord
   
case 'BEAM'
   disp('Data in Beam coordinates is being transformed to Earth')
   %if 0
   if running(batch)
      dlgFile = get(batch);
   	eval(['dlgFile = ' dlgFile ';'])      
   end

 
   cur = runbm2g(adcpFile, ADCPtype, dlgFile);
   %end
   %load cur.mat
   close(B)
   B = netcdf(adcpFile ); %added 09-Apr-2003. Now runbm2g history comment will be added to epic file.
   
   
case 'EARTH'
   disp('Data is in Earth coordinates')
   for ii = 1:4;
      vel{ii} = B{['vel' int2str(ii)]};
   end
   
   %need to adjust for magnetic declination
   u = vel{1}(:);
   v = vel{2}(:);
   magnetic = B{'Hdg'}.heading_bias(:);
   theta = -1*magnetic;
   [ur,vr] = uv_rotate(u,v,theta); 
   %need to reset theFillValue
   urmax = max(max(ur));  vrmax = max(max(vr));
   ur(ur == urmax) = theFillValue;
   vr(vr == vrmax) = theFillValue;
   %fill in rotated velocity values
   [vel{1},vel{2}] = deal(ur,vr);
      
	cur = cell(size(vel));   % Output currents and error.
	p = zeros(4, bin);
   for ii=1:ensemble
		for k = 1:4
  	 		p(k, :) = vel{k}(ii, :);	
			cur{k}(ii, :) = p(k, :);
   	end
      if ~rem(ii,100), 
      disp(sprintf('%d ensembles copied',ii)), 
   	end
   end
end

%scale as cm/sec
q = zeros(ensemble, bin);
for k =1:4;
  	q(:, :) = cur{k}(:, :);	
   q(q == theFillValue) = nan;
   q = q./10;
   q(isnan(q)) = theFillValue;
   cur{k}(:,:) = q;
 end

%Get the mins and maxes for later
for k=1:4
   good_cur=(cur{k}(:,:) ~= theFillValue);
   scur=cur{k}(logical(good_cur));
   minsc(k) = min(min(scur));
   maxsc(k) = max(max(scur));
end
clear p q good_cur scur

% Check for fill values.
   
   for k=1:4;
		TF = isnan(cur{k}(:,:));
		MTF =(max(max(TF)));
		if MTF
   		fill_flag = 1;
		else
   		fill_flag = 0;
	   end
	end

   
%**********************************************************************
% Create the new output file.
G = netcdf(epDataFile,'clobber');

%Global Attributes
%copy the globals from the Beam File
%figure out what needs to be deleted, changed, or added to the final files
%delete the following variables
disp('Modifying the global attributes');
A=att(B);
Anames=ncnames(A);
AN=length(Anames);

for nn=1:AN-1
   if isequal(Anames{AN-nn},'sensor_configuration');
      A(AN-nn) = [];
   elseif isequal(Anames{AN-nn},'transducer_attached');
       A(AN-nn) = [];
    elseif isequal(Anames{AN-nn},'simulated_data');
       A(AN-nn) = [];
    elseif isequal(Anames{AN-nn},'beams_in_velocity_calculation');
       A(AN-nn) = [];
    elseif isequal(Anames{AN-nn},'profiling_mode');
       A(AN-nn) = [];
    elseif isequal(Anames{AN-nn},'code_repetitions');
       A(AN-nn) = [];
    elseif isequal(Anames{AN-nn},'transmit_lag_distance');
       A(AN-nn) = [];
    elseif isequal(Anames{AN-nn},'transform');
       A(AN-nn) = [];
       %added 12/15/04 (SDR)
    elseif isequal(Anames{AN-nn},'Sound_speed_computed_from_ED_ES_ET');
       A(AN-nn) = [];
    elseif isequal(Anames{AN-nn},'ED_taken_from_depth_sensor');
       A(AN-nn) = [];
    elseif isequal(Anames{AN-nn},'EH_taken_from_xducer_heading_sensor');
       A(AN-nn) = [];
    elseif isequal(Anames{AN-nn},'EP_taken_from_xducer_pitch_sensor');
       A(AN-nn) = [];
    elseif isequal(Anames{AN-nn},'ER_taken_from_xducer_roll_sensor');
       A(AN-nn) = [];
    elseif isequal(Anames{AN-nn},'ES_derived_from_conductivity_sensor');
       A(AN-nn) = [];
    elseif isequal(Anames{AN-nn},'ET_taken_from_temperature_sensor');
       A(AN-nn) = [];
    elseif isequal(Anames{AN-nn},'depth_sensor');
       A(AN-nn) = [];
    elseif isequal(Anames{AN-nn},'heading_sensor');
       A(AN-nn) = [];
    elseif isequal(Anames{AN-nn},'pitch_sensor');
       A(AN-nn) = [];
    elseif isequal(Anames{AN-nn},'roll_sensor');
       A(AN-nn) = [];
    elseif isequal(Anames{AN-nn},'conductivity_sensor');
       A(AN-nn) = [];
    elseif isequal(Anames{AN-nn},'temperature_sensor');
       A(AN-nn) = [];
    end
 end
 
 
mm=1;

while mm < AN-1
	notenum = ['NOTE_',int2str(mm)];
	Anames=ncnames(A);
 	AN=length(Anames);
	for nn=1:AN-1
      if isequal(Anames{AN-nn},notenum); 
         A(AN-nn) = [];
      end
   end
	mm=mm+1;
end   
       
%put the ones that are left into new file (G)
G < A;

%change the following attributes and variables
G.CREATION_DATE = datestr(now,0);

%RENAME A FEW
moor = G.Mooring_number;
moor = rename(moor,'MOORING');
pulse = G.transmit_pulse_length;
pulse = rename(pulse,'transmit_pulse_length_cm');

%Define some new ones for Epic compatability
G.transform = 'EARTH';
G.DATA_TYPE = 'ADCP';
G.DATA_SUBTYPE = 'MOORED';
G.DATA_ORIGIN = 'USGS WHFS Sed Trans Group';
G.COORD_SYSTEM = 'GEOGRAPHIC';
G.WATER_MASS = ncchar('?');
G.POS_CONST = nclong(0);  %1 if consistent
G.DEPTH_CONST = nclong(0);  %1 if consistent
G.WATER_DEPTH = wdepth;
G.DRIFTER = nclong(0);

G.VAR_FILL = theFillValue;

if running(batch)
   experiment = get(batch); 
   eval(['experiment = ' experiment ';'])
   project = get(batch);
   eval(['project = ' project ';'])
   descript = get(batch);
   eval(['descript = ' descript ';'])
   long = get(batch);
   eval(['long = ' long ';'])
   lonUnits = get(batch);
   eval(['lonUnits = ' lonUnits ';'])
   latit = get(batch);
   eval(['latit = ' latit ';'])
   latUnits = get(batch);
   eval(['latUnits = ' latUnits ';'])
   cmnt = get(batch);  
	eval(['cmnt = ' cmnt ';'])
else
   
% NEED TO ASK FOR the following
%  Data_Collection_Information.Mooring = {G.MOORING(:)};
%  Data_Collection_Information.Experiment = {{'Mass Bay Longterm', 'Globec', 'Southern California','Monterey Bay','GEOPROBE','ECOHAB'...
%              'Orange County Sanitation District','OTHER'}};   % 1 or 2.
%  Data_Collection_Information.Project = {{'MWRA', 'NOAA', 'WCMG','WHFC','NOPP','OTHER'}} ;  
%  Data_Collection_Information.descript = {{'Boston Buoy B ADCP','Scituate Buoy ADCP','Georges Bank ADCP',...
%              'Great South Channel ADCP','Socal ADCP','Montery ADCP','Santa Monica Bay ADCP','Casco Bay ADCP','Jordan Basin ADCP',...
%           'Gulf of Maine ADCP','OTHER ADCP','LA Shelf ADCP M03','LA Shelf ADCP M05','LA Shelf ADCP M07','LA Shelf ADCP M11'}};
%  Data_Collection_Information.comments = {' '};
% 
%  	Data_Collection_Information.longitude.value = {0};
%    Data_Collection_Information.longitude.format = {'decimal degrees'};
%    Data_Collection_Information.longitude.units = {'degree_east','degree_west'};
% 
%  	Data_Collection_Information.latitude.value = {0};
%    Data_Collection_Information.longitude.format = {'decimal degrees'};
%    Data_Collection_Information.latitude.units = {'degree_north','degree_south'};
%    
%  
%  Data_Collection_Information = uigetinfo(Data_Collection_Information);
%  
%  % Revised so getinfo will work with Matlab 6.0, r. 12 (ALR)
%  experiment= getinfo(Data_Collection_Information,'Experiment');
%  project = getinfo(Data_Collection_Information,'Project');
%  descript = getinfo(Data_Collection_Information,'descript');
%  longInfo = getinfo(Data_Collection_Information,'longitude');
%  long = getinfo(longInfo,'value');
%  lonUnits = getinfo(longInfo,'units');
%  latitInfo = getinfo(Data_Collection_Information,'latitude');
%  latit = getinfo(latitInfo,'value');
%  latUnits=getinfo(latitInfo,'units');
%  cmnt = getinfo(Data_Collection_Information,'comments');
%  


%  %added for easier data entry
	prompt  = {...
    'Experiment:                                    ',...
    'Description:                                   ',...
	'Project:                                       ',...
	'Comments:                                      ',...
    'Longitude(decimal degrees):                    ',...
    'Units:                                         ',...
    'Latitude(decimal degrees):                     ',...
    'Units:                                         '};
      
	def     = {'','','','none','','degree_east','','degree_north'};
	title   = ['Data Collection Information: ',G.MOORING(:)];
	lineNo  = 1;
	dlgresult  = inputdlg(prompt,title,lineNo,def,'on');
	experiment = dlgresult{1};
    project = dlgresult{2};
   	descript = dlgresult{3};
	cmnt = dlgresult{4};
    long = str2num(dlgresult{5});
    lonUnits = dlgresult{6};
    latit = str2num(dlgresult{7});
    latUnits = dlgresult{8};
    
    if isempty(lonUnits), lonUnits = 'not specified'; end
    if isempty(latUnits), latUnits = 'not specified'; end 
    if isempty(long), long = theFillValue; end
    if isempty(latit), latit = theFillValue; end

 
end

 G.EXPERIMENT =experiment;
 G.PROJECT = project;
 G.DESCRIPT = descript;
 G.longitude = long;
 G.latitude = latit;
 G.DATA_CMNT = cmnt;  

%Need to calculate or derive the following
G.FILL_FLAG=nclong(fill_flag);
G.COMPOSITE=nclong(0);
G.VAR_DESC='u:v:w:Werr:AGC:PGd:Tx';

%calculate dt** this section removed FH 10 May 2000 
%tim=B{'TIM'};
%gt=gregorian(tim(:));
%dt=diff(gt);
%delta=(dt(1,4).*60)+(dt(1,5))+(dt(1,6)./60);

G.DELTA_T=ncchar(num2str(delta));
G.start_time = start_time;
G.stop_time = stop_time;

magnetic = B{'Hdg'}.heading_bias(:);
if isempty(magnetic)
   magnetic = B.heading_bias(:)/100;
end
G.magnetic_variation = magnetic;

%define the dimensions
G('time') = 0;
G('depth') = length(B{'D'});
G('lon') = 1;
G('lat') = 1;


%Variables and Attributes
disp('Defining variables and their attributes')
disp('...Defining "time"')

G{'time'} = nclong('time');
G{'time'}.FORTRAN_format = ncchar('F10.2');
G{'time'}.units = ncchar('True Julian Day');
G{'time'}.type = ncchar('UNEVEN');
G{'time'}.epic_code = nclong(624);
%G{'time'}.FillValue_ = theFillValue;


disp('...Defining "time2"')
G{'time2'} = nclong('time') ;
G{'time2'}.FORTRAN_format = ncchar('F10.2');
G{'time2'}.units = ncchar('msec since 0:00 GMT');
G{'time2'}.type = ncchar('UNEVEN');
G{'time2'}.epic_code = nclong(624);
%G{'time2'}.FillValue_ = theFillValue;


disp('...Defining "depth"')
G{'depth'} = ncfloat('depth'); 
G{'depth'}.FORTRAN_format = ncchar('F10.2');
G{'depth'}.units = ncchar('m');
G{'depth'}.type = ncchar('EVEN');
G{'depth'}.epic_code = nclong(3);
G{'depth'}.long_name = ncchar('DEPTH (m)');
G{'depth'}.blanking_distance = B{'D'}.blanking_distance(:);
G{'depth'}.bin_size = B{'D'}.bin_size(:);
G{'depth'}.xducer_offset_from_bottom = B{'D'}.xducer_offset_from_bottom(:);
G{'depth'}.FillValue_ = theFillValue;
G{'depth'}.NOTE = ncchar(dnote);

disp('...Defining "lon"')
G{'lon'} = ncfloat('lon'); %% 1 element.
G{'lon'}.FORTRAN_format = ncchar('f10.4');
G{'lon'}.units = ncchar(lonUnits);
G{'lon'}.type = ncchar('EVEN');
G{'lon'}.epic_code = nclong(502);
G{'lon'}.name = ncchar('LON');
G{'lon'}.long_name = ncchar('LONGITUDE');
G{'lon'}.generic_name = ncchar('lon');
G{'lon'}.FillValue_ = theFillValue;

disp('...Defining "lat"')
G{'lat'} = ncfloat('lat'); %% 1 element.
G{'lat'}.FORTRAN_format = ncchar('F10.2');
G{'lat'}.units = ncchar(latUnits);
G{'lat'}.type = ncchar('EVEN');
G{'lat'}.epic_code = nclong(500);
G{'lat'}.name = ncchar('LAT');
G{'lat'}.long_name = ncchar('LATITUDE');
G{'lat'}.generic_name = ncchar('lat');
G{'lat'}.FillValue_ = theFillValue;

disp('...Defining "u_1205"')
G{'u_1205'} = ncfloat('time', 'depth', 'lat', 'lon');
G{'u_1205'}.name = ncchar('u');
G{'u_1205'}.long_name = ncchar('Eastward Velocity');
G{'u_1205'}.generic_name = ncchar('u');
G{'u_1205'}.FORTRAN_format = ncchar(' ');
G{'u_1205'}.units = ncchar('cm/s');
G{'u_1205'}.epic_code = nclong(1205);
G{'u_1205'}.sensor_type = B.INST_TYPE(:);
G{'u_1205'}.sensor_depth = wdepth - xducer_off;
G{'u_1205'}.serial_number = nclong(serial);

G{'u_1205'}.minimum = ncfloat(minsc(1));
G{'u_1205'}.maximum = ncfloat(maxsc(1));
%Not sure this is the right valid range, but what Fran had
G{'u_1205'}.valid_range = ncfloat([-1000 1000]);
G{'u_1205'}.FillValue_ = theFillValue;


disp('...Defining "v_1206"')
G{'v_1206'} = ncfloat('time', 'depth', 'lat', 'lon'); 
G{'v_1206'}.name = ncchar('v');
G{'v_1206'}.long_name = ncchar('Northward Velocity');
G{'v_1206'}.generic_name = ncchar('v');
G{'v_1206'}.FORTRAN_format = ncchar(' ');
G{'v_1206'}.units = ncchar('cm/s');
G{'v_1206'}.epic_code = nclong(1206);
G{'v_1206'}.sensor_type = B.INST_TYPE(:);
G{'v_1206'}.sensor_depth = wdepth - xducer_off;
G{'v_1206'}.serial_number = nclong(serial);
G{'v_1206'}.minimum = ncfloat(minsc(2));
G{'v_1206'}.maximum = ncfloat(maxsc(2));
G{'v_1206'}.valid_range = ncfloat([-1000 1000]);
G{'v_1206'}.FillValue_ = theFillValue;


disp('...Defining "w_1204"')
G{'w_1204'} = ncfloat('time', 'depth', 'lat', 'lon'); 
G{'w_1204'}.name = ncchar('w');
G{'w_1204'}.long_name = ncchar('Vertical Velocity');
G{'w_1204'}.generic_name = ncchar('w');
G{'w_1204'}.FORTRAN_format = ncchar(' ');
G{'w_1204'}.units = ncchar('cm/s');
G{'w_1204'}.epic_code = nclong(1204);
G{'w_1204'}.sensor_type = B.INST_TYPE(:);
G{'w_1204'}.sensor_depth = wdepth - xducer_off;
G{'w_1204'}.serial_number = nclong(serial);
G{'w_1204'}.minimum = ncfloat(minsc(3));
G{'w_1204'}.maximum = ncfloat(maxsc(3));
G{'w_1204'}.valid_range = ncfloat([-1000 1000]);
G{'w_1204'}.FillValue_ = theFillValue;

disp('...Defining "Werr_1201"')
G{'Werr_1201'} = ncfloat('time', 'depth', 'lat', 'lon'); 
G{'Werr_1201'}.name = ncchar('Werr');
G{'Werr_1201'}.long_name = ncchar('Error Velocity');
G{'Werr_1201'}.generic_name = ncchar('w');
G{'Werr_1201'}.FORTRAN_format = ncchar('F8.1');
G{'Werr_1201'}.units = ncchar('cm/s');
G{'Werr_1201'}.epic_code = nclong(1201);
G{'Werr_1201'}.sensor_type = B.INST_TYPE(:);
G{'Werr_1201'}.sensor_depth = wdepth - xducer_off;
G{'Werr_1201'}.serial_number = nclong(serial);
G{'Werr_1201'}.minimum = ncfloat(minsc(4));
G{'Werr_1201'}.maximum = ncfloat(maxsc(4));
G{'Werr_1201'}.valid_range = B.error_velocity_threshold(:);
G{'Werr_1201'}.FillValue_ = theFillValue;


disp('...Defining "AGC_1202"')
G{'AGC_1202'} = ncfloat('time', 'depth', 'lat', 'lon'); 
G{'AGC_1202'}.name = ncchar('AGC');
G{'AGC_1202'}.long_name = ncchar('Average Echo Intensity (AGC)');
G{'AGC_1202'}.generic_name = ncchar('AGC');
G{'AGC_1202'}.FORTRAN_format = ncchar('F5.1');
G{'AGC_1202'}.units = ncchar('counts');
G{'AGC_1202'}.epic_code = nclong(1202);
G{'AGC_1202'}.sensor_type = B.INST_TYPE(:);
G{'AGC_1202'}.sensor_depth = wdepth - xducer_off;
G{'AGC_1202'}.serial_number = nclong(serial);
G{'AGC_1202'}.norm_factor = B{'AGC1'}.norm_factor(:);
G{'AGC_1202'}.NOTE = ncchar('normalization to db');
G{'AGC_1202'}.minimum = ncfloat(min(min(iavg)));
G{'AGC_1202'}.maximum = ncfloat(max(max(iavg)));
G{'AGC_1202'}.valid_range = B.false_target_reject_values(:);
G{'AGC_1202'}.FillValue_ = theFillValue;


disp('...Defining "PGd_1203"')
G{'PGd_1203'} = ncfloat('time', 'depth', 'lat', 'lon');
G{'PGd_1203'}.name = ncchar('PGd');
G{'PGd_1203'}.long_name = ncchar('Percent Good Pings');
G{'PGd_1203'}.generic_name = ncchar('PGd');
G{'PGd_1203'}.FORTRAN_format = ncchar(' ');
G{'PGd_1203'}.units = ncchar('counts');
G{'PGd_1203'}.epic_code = nclong(1203);
G{'PGd_1203'}.sensor_type = B.INST_TYPE(:);
G{'PGd_1203'}.sensor_depth = wdepth - xducer_off;
G{'PGd_1203'}.serial_number = nclong(serial);
G{'PGd_1203'}.minimum = ncfloat(min(min(pavg)));
G{'PGd_1203'}.maximum = ncfloat(max(max(pavg)));
G{'PGd_1203'}.valid_range = B.minmax_percent_good(:);
G{'PGd_1203'}.FillValue_ = theFillValue;

disp('...Defining "hght_18"')
G{'hght_18'} = ncfloat('time', 'lat', 'lon'); 
G{'hght_18'}.name = ncchar('hght');
G{'hght_18'}.long_name = ncchar('height of sea surface');
G{'hght_18'}.generic_name = ncchar('height');
G{'hght_18'}.FORTRAN_format = ncchar('f10.2');
G{'hght_18'}.units = ncchar('m');
G{'hght_18'}.epic_code = nclong(18);
G{'hght_18'}.sensor_depth = wdepth - xducer_off;
G{'hght_18'}.minimum = ncfloat(min(height));
G{'hght_18'}.maximum = ncfloat(max(height));
G{'hght_18'}.serial_number = nclong(serial);
G{'hght_18'}.valid_range = ncfloat([0 1000]);
G{'hght_18'}.FillValue_ = theFillValue;
G{'hght_18'}.NOTE = ncchar('height of sea surface relative to transducer head from surface.exe');

disp('...Defining "Tx_1211"')
G{'Tx_1211'} = ncfloat('time', 'lat', 'lon');
G{'Tx_1211'}.name = ncchar('Tx');
G{'Tx_1211'}.long_name = ncchar('ADCP Transducer Temp.');
G{'Tx_1211'}.generic_name = ncchar('temp');
G{'Tx_1211'}.units = ncchar('degrees.C');
G{'Tx_1211'}.epic_code = nclong(1211);
G{'Tx_1211'}.sensor_type = B.INST_TYPE(:);
G{'Tx_1211'}.sensor_depth = wdepth - xducer_off;
G{'Tx_1211'}.serial_number = nclong(serial);
G{'Tx_1211'}.minimum = ncfloat(min(tempC));
G{'Tx_1211'}.maximum = ncfloat(max(tempC));
G{'Tx_1211'}.valid_range = B{'Tx'}.valid_range(:);
G{'Tx_1211'}.FillValue_ = theFillValue;

disp('...Defining "P_4"')
G{'P_4'} = ncfloat('time', 'lat', 'lon') 
G{'P_4'}.name = ncchar('P');
G{'P_4'}.long_name = ncchar('PRESSURE (PASCALS)');
G{'P_4'}.generic_name = ncchar('depth');
G{'P_4'}.units = ncchar('Pa');
G{'P_4'}.epic_code = nclong(4);
G{'P_4'}.sensor_type = B.INST_TYPE(:);
G{'P_4'}.sensor_depth = wdepth - xducer_off;
G{'P_4'}.serial_number = nclong(serial);
G{'P_4'}.minimum = ncfloat(min(Press));
G{'P_4'}.maximum = ncfloat(max(Press));
G{'P_4'}.valid_range = B{'Pressure'}.valid_range(:);
G{'P_4'}.FillValue_ = theFillValue;
G{'P_4'}.NOTE = ncchar('Pressure of the water at the transducer head relative to one atmosphere (sea level)');

endef(G)


%Put in the Data
m=ensemble;
n=bin;
disp(['Copying data to ' epDataFile]);
disp('... time')
G{'time'}(1:m)= Time;
disp('... time2')
G{'time2'}(1:m) = Time2;
disp('... lat')
G{'lat'}(1) = G.latitude(:);
disp('... lon')
G{'lon'}(1) = G.longitude(:);
disp('... depth')
G{'depth'}(1:n) = depth;
disp('... u_1205')
G{'u_1205'}(1:m, 1:n, 1, 1) = cur{1}(:,:);
disp('... v_1206')
G{'v_1206'}(1:m, 1:n, 1, 1) = cur{2}(:,:);
disp('... w_1204')
G{'w_1204'}(1:m, 1:n, 1, 1) = cur{3}(:,:);
disp('... Werr_1201')
G{'Werr_1201'}(1:m, 1:n, 1, 1) = cur{4}(:,:);
disp('... AGC_1202')
G{'AGC_1202'}(1:m, 1:n, 1, 1) = iavg;
disp('... PGd_1203')
G{'PGd_1203'}(1:m, 1:n, 1, 1) = pavg;
disp('...hght_18')
G{'hght_18' }(1:m, 1, 1) = height;
disp('... Tx_1211')
G{'Tx_1211'}(1:m, 1, 1) = tempC;
disp('... P_4')
G{'P_4'}(1:m, 1, 1) = Press;

      close(B)
      close(G)
      
      
thecomment = sprintf('%s\n','Written to an EPIC standard data file by adcp2ep.m (version 1.1)');
history(epDataFile,thecomment);
