function [nMissEns] = fixEns(rawcdf,theFilledFile)

% function  missing = fixEns(rawcdf, theFilledFile)
% Checks for missing ensembles in the ADCP data and places place holders with
% fill values for any missing ensembles in the netCDF files
% If only rawcdf is specified, then only checks for missing ensembles
%
% INPUTS:
%       rawcdf = the raw netCDF file created by rdi2cdf.m
%       theFilledFile = the name of the file created if missing ensembles are found and filled
%
% OUTPUTS:
%       nMissENs = the number of missing ensembles found
%
% Written by Andree L. Ramsey
% for the U.S. Geological Survey
% Coastal and Marine Geology Program
% Woods Hole, MA
% http://woodshole.er.usgs.gov/
% Please report bugs to aramsey@usgs.gov

% updated 10-Jan-2003 (ALR) added ability to run without having to give inputs names and added information
%   to the description
% updated 20-Sep-2001 (ALR) to run on PC and to print missing ensemble numbers to history attribute
% Version 2.0 updated by Marinna Martini
% Version 1.1
% changed to look for PD12 data and act appropriately, 7/12/01 Marinna Martini
% updated 28-Dec-2000 09:27:25 - added linefeed to comment/history attribute (ALR)
% updated 15-Dec-2000 - adjusted so will work whether height variable exists or not
% Beta version 06-Dec-2000 - created

Mname=mfilename;
disp('')
disp([Mname ' is currently running version 2.0']);


if nargin < 1, rawcdf = ''; end
if nargin < 2, theFilledFile = ''; end

if isempty(rawcdf), rawcdf = '*'; end
if isempty(theFilledFile), theFilledFile = '*'; end

% Get ADCP filename.
if any(rawcdf == '*')
	[theFile, thePath] = uigetfile(rawcdf, 'Select Netcdf ADCP File:');
	if ~any(theFile), return, end
	if thePath(end) ~= filesep, thePath(end+1) = filesep; end
	rawcdf = [thePath theFile];
end


%First step is to determine if there are any missing ensemble numbers
a=netcdf(rawcdf);
Rec=a{'Rec'}(:);
nRec=length(Rec);
firstRec=Rec(1);
lastRec=Rec(end);
nEns=lastRec-(firstRec-1); % why is 1 subtracted here?
nMissEns=nEns-nRec;
if nMissEns == 0,
   disp('Yippie...there are no missing ensembles!! :) :) :) :) :) :) :) ')
   ncclose
   return
end

disp(sprintf('Rats...there are %d missing ensemble',nMissEns))
disp('')
disp('FixEns.m will place fill values in for missing ensemble numbers')
disp('This may take a while')

% detect PD12 data.  This is important because PD12 does not contain all variables
PD12=0;
data_subtype = a.DATA_SUBTYPE(:);
if ~isempty(data_subtype),
    disp([mfilename,': data type = ',data_subtype])
    if findstr(data_subtype, 'PD12'), 
        disp([mfilename,': trapping for PD12 data']);
        PD12=1;
    end
end

% reconstruct the time and record indeces
oldTIM = a{'TIM'}(:);
newRec = firstRec:lastRec;
newTIM = interp1(Rec, oldTIM, newRec);

disp(['Size of oldRec = ',int2str(length(Rec))])
disp(['Size of newRec = ',int2str(length(newRec))])

% make a map to where the gaps were for padding later
gapmask = zeros(nEns, 1);  % make an array as long as the number of ensembles that should be present
goodidx = Rec-firstRec+1;   % an index into this array for the ensembles received
gapmask(goodidx) = ones(length(goodidx),1); % set those array positions to TRUE
MissEnsNos = find(gapmask == 0);
% now newRec(gapmask) should be equal to oldRec

fillV=fillval(a{'D'});  % what we will pad the missing ensembles with

% now, create a new, expanded netCDF file
c = netcdf(theFilledFile,'clobber');
% write new, expanded dimensions
%c('ensemble') = nEns;   % this expands to the number of ensembles there SHOULD be
c('bin') = ncsize(a('bin'));
% copy the global attributes
c < att(a);
% copy the variables, this moves the definitions and attributes to the new file, 
% but the data are now out of sequence
c < var(a);

% now must fill in the data itself while accounting for the new gaps
% TIM and Rec are already fixed
c{'TIM'}(:)=newTIM;
c{'Rec'}(:)=newRec; 
% now fix data that's present in all ADCP file type
% 1D variables first
datatypes = {'Hdg'; 'Ptch'; 'Roll'; 'Tx'; 'height'; 'sv'; 'xmitc'; 'xmitv'; 'dac';...
        'VDD3'; 'VDD1'; 'VDC'; 'height'}; 
% note that D is not listed here... it does not need to be fixed
for i=1:length(datatypes),
    disp(datatypes{i})
    if c{datatypes{i}}, % if they exist in this file.  Thus traps for variables missing from PD12 data        
        newData = ones(length(newRec),1).*fillV;
        newData(goodidx) = a{datatypes{i}}(:); % fill in the good data
        c{datatypes{i}}(:) = newData;
    end
end
% now 2D variables
datatypes = {'vel1'; 'vel2'; 'vel3'; 'vel4'};
% must do it this way for 2D variables.  For some reason, if c{datatypes{i}} doesn't work here
if ~PD12,
    datatypes = {'vel1'; 'vel2'; 'vel3'; 'vel4'; 'cor1'; 'cor2'; 'cor3'; 'cor4';...
        'AGC1'; 'AGC2'; 'AGC3'; 'AGC4'; 'PGd1'; 'PGd2'; 'PGd3'; 'PGd4'};
end
nbins = length(c{'D'}(:));
for i=1:length(datatypes),
    disp(datatypes{i})
    newData = ones(length(newRec),nbins).*fillV;
    newData(goodidx,:) = a{datatypes{i}}(:,:); % fill in the good data
    c{datatypes{i}}(:,:) = newData;
end

disp('Fill values have been filled in as place holders for missing ensemble numbers')

ncclose

%Last, need a commnet in the history field
thecomment1=sprintf...
   ('%s\n','The missing ensemble numbers that were filled with fill values using fixEns.m were: ');
thecomment2=(sprintf(' %d;',MissEnsNos));
thecomment3=(sprintf('%s\n',''));   %A line feed to separate the previous comment
thecomment=[thecomment1 thecomment2 thecomment3];
history(theFilledFile,thecomment);





